"""The color item gives the user the posibility to choice a colour."""
# Copyright (C) 2012 S. Daniel Francis <francis@sugarlabs.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

import logging
logger = logging.getLogger('option')

import gobject
import gtk

from colors import color2string
from item import Item


class ColorItem(Item):
    """Color Selecting interface.
    In Sugar it creates a sugar.graphics.colorbutton.ColorToolButton.
    In other desktops it's a sweetener.item.Item connected to a
    gtk.ColorSelectionDialog.
    """
    __gsignals__ = {'updated': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                                (gobject.TYPE_STRING,))}

    def __init__(self, parent=None, important=False):
        """Constructor.
        parent -- A window where focus the dialog or None.
        important -- bool , if it's True, in desktops a ToolItem will be
        appended to the Toolbar.
        """
        Item.__init__(self, gtk.STOCK_SELECT_COLOR, important)
        self.parent = parent
        self.color = '#FFFFFF'

    def set_color(self, color):
        """color -- RGB color"""
        self.color = color

    def _color_changed_cb(self, widget):
        color_gdk = widget.get_current_color()
        self.color = color2string(color_gdk)
        self.emit('updated', self.color)

    def do_activate(self):
        if self.tooltip:
            title = self.tooltip
        else:
            title = gtk.stock_lookup(self.stock_id)[1]
        dialog = gtk.ColorSelectionDialog(title)
        dialog.set_transient_for(self.parent)
        color_selection = dialog.get_color_selection()
        color_selection.connect('color-changed', self._color_changed_cb)
        color_selection.set_current_color(gtk.gdk.color_parse(self.color))
        dialog.props.cancel_button.hide()
        #dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT)
        dialog.run()
        dialog.destroy()
        self.emit('updated', self.color)
